using System;
using System.IO;
using System.Collections;

class MyApp
{
    static void Main (string[] args)
    {
        // Make sure a file name was entered on the command line
        if (args.Length == 0) {
            Console.WriteLine ("Error: Missing file name");
            return;
        }

        StreamReader reader = null;
        Hashtable table = new Hashtable ();

        try {
            // Iterate through the file a word at a time, creating
            // an entry in the Hashtable for each unique word found
            // and incrementing the count for each word that's
            // encountered again
            reader = new StreamReader (args[0]);

            for (string line = reader.ReadLine (); line != null;
                line = reader.ReadLine ()) {
                string[] words = GetWords (line);

                foreach (string word in words) {
                    string iword = word.ToLower ();
                    if (table.ContainsKey (iword))
                        table[iword] = (int) table[iword] + 1;
                    else
                        table[iword] = 1;
                }
            }

            // Sort the Hashtable entries using a SortedList
            SortedList list = new SortedList (table);

            // Display the results
            Console.WriteLine ("{0} unique words found in {1}",
                table.Count, args[0]);

            foreach (DictionaryEntry entry in list)
                Console.WriteLine ("{0} ({1})",
                    entry.Key, entry.Value);
        }
        catch (Exception e) {
            Console.WriteLine (e.Message);
        }
        finally {
            if (reader != null)
                reader.Close ();
        }
    }

    static string[] GetWords (string line)
    {
        // Create an ArrayList to hold the intermediate results
        ArrayList al = new ArrayList ();

        // Parse the words from the line and add them to the ArrayList
        int i = 0;
        string word;
        char[] characters = line.ToCharArray ();

        while ((word = GetNextWord (line, characters, ref i)) != null)
            al.Add (word);

        // Return a static array that is equivalent to the ArrayList
        string[] words = new string[al.Count];
        al.CopyTo (words);
        return words;
    }

    static string GetNextWord (string line, char[] characters,
        ref int i)
    {
        // Find the beginning of the next word
        while (i < characters.Length &&
            !Char.IsLetterOrDigit (characters[i]))
            i++;

        if (i == characters.Length)
            return null;

        int start = i;

        // Find the end of the word
        while (i < characters.Length &&
            Char.IsLetterOrDigit (characters[i]))
          i++;

        // Return the word
        return line.Substring (start, i - start);
    }
}
